@@ -1,3 +1,11 @@
+suppress_bootstrap_assumption_warning <- function(expr) {
+  withCallingHandlers(expr, warning = function(w) {
+    if (grepl("injects replicate analysis weights", conditionMessage(w), fixed = TRUE)) {
+      invokeRestart("muffleWarning")
+    }
+  })
+}
+
 test_that("svyrep.design is rejected with clear error", {
   skip_if_not_installed("survey")
   skip_if_not_installed("svrep")
@@ -8,7 +16,6 @@ test_that("svyrep.design is rejected with clear error", {
     data = apistrat, fpc = ~fpc
   )
 
-# Create replicate design
   drep <- svrep::as_bootstrap_design(dstrat, replicates = 10)
 
   estimator <- function(data, ...) {
@@ -16,13 +23,11 @@ test_that("svyrep.design is rejected with clear error", {
     list(y_hat = as.numeric(est), converged = TRUE)
   }
 
-# Should error with informative message
   expect_error(
     bootstrap_variance(
       drep, estimator,
       point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 10,
-      bootstrap_cores = 1
+      bootstrap_reps = 10
     ),
     "replicate design|svyrep"
   )
@@ -45,27 +50,63 @@ test_that("replicate count mismatch warns but proceeds", {
 
   future::plan(future::sequential)
 
-# Request a large number - may produce fewer
-# This test verifies we warn and proceed (not error)
   result <- tryCatch(
     suppressWarnings(
       bootstrap_variance(
         dstrat, estimator,
         point_estimate = mean(apistrat$api00),
-        bootstrap_reps = 200,
-        bootstrap_cores = 1
+        bootstrap_reps = 200
       )
     ),
     error = function(e) list(error = TRUE, message = e$message)
   )
 
-# Should succeed (not error)
   expect_false(isTRUE(result$error))
   expect_true(is.finite(result$variance))
   expect_true(result$variance > 0)
   expect_true(length(result$replicates) >= 2)
 })
 
+test_that("survey bootstrap errors when replicate design has too few replicates", {
+  skip_if_not_installed("survey")
+  skip_if_not_installed("svrep")
+
+  df <- data.frame(y = 1:4, w = 1)
+  design <- survey::svydesign(ids = ~1, data = df, weights = ~w)
+
+  estimator <- function(data, ...) {
+    est <- survey::svymean(~y, data)
+    list(y_hat = as.numeric(est), converged = TRUE)
+  }
+
+  testthat::local_mocked_bindings(
+    as_bootstrap_design = function(design, replicates, ...) {
+      repw <- matrix(stats::weights(design), ncol = 1)
+      survey::svrepdesign(
+        variables = design$variables,
+        repweights = repw,
+        weights = stats::weights(design),
+        type = "bootstrap",
+        scale = 1,
+        rscales = 1,
+        combined.weights = FALSE,
+        mse = FALSE
+      )
+    },
+    .package = "svrep"
+  )
+
+  expect_error(
+    bootstrap_variance(
+      design, estimator,
+      point_estimate = mean(df$y),
+      bootstrap_reps = 10
+    ),
+    "at least 2 replicates",
+    fixed = TRUE
+  )
+})
+
 test_that("survey NA policy 'strict' shows detailed error", {
   skip_if_not_installed("survey")
   skip_if_not_installed("svrep")
@@ -92,12 +133,13 @@ test_that("survey NA policy 'strict' shows detailed error", {
   future::plan(future::sequential)
 
   err <- tryCatch(
-    bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 10,
-      bootstrap_cores = 1,
-      survey_na_policy = "strict"
+    suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 10,
+        survey_na_policy = "strict"
+      )
     ),
     error = function(e) e$message
   )
@@ -118,7 +160,7 @@ test_that("survey NA policy 'omit' handles failures correctly", {
     data = apistrat, fpc = ~fpc
   )
 
-# Controlled failure pattern
+# Controlled failure
   fail_at <- c(3, 7, 12)
   counter <- 0
   estimator <- function(data, ...) {
@@ -134,12 +176,13 @@ test_that("survey NA policy 'omit' handles failures correctly", {
   future::plan(future::sequential)
 
   expect_warning(
-    res <- bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 15,
-      bootstrap_cores = 1,
-      survey_na_policy = "omit"
+    res <- suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 15,
+        survey_na_policy = "omit"
+      )
     ),
     "3/15.*failed.*omitted"
   )
@@ -170,12 +213,13 @@ test_that("survey NA policy 'omit' requires at least 2 successes", {
   future::plan(future::sequential)
 
   expect_error(
-    bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 10,
-      bootstrap_cores = 1,
-      survey_na_policy = "omit"
+    suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 10,
+        survey_na_policy = "omit"
+      )
     ),
     "Too few successful"
   )
@@ -209,16 +253,17 @@ test_that("survey NA policy 'omit' shows failure pattern", {
   warn_msg <- NULL
   res <- tryCatch(
     withCallingHandlers(
-      bootstrap_variance(
-        dstrat, estimator,
-        point_estimate = mean(apistrat$api00),
-        bootstrap_reps = 20,
-        bootstrap_cores = 1,
-        survey_na_policy = "omit"
+      suppress_bootstrap_assumption_warning(
+        bootstrap_variance(
+          dstrat, estimator,
+          point_estimate = mean(apistrat$api00),
+          bootstrap_reps = 20,
+          survey_na_policy = "omit"
+        )
       ),
-      warning = function(w) {
-        warn_msg <<- conditionMessage(w)
-        invokeRestart("muffleWarning")
+    warning = function(w) {
+      warn_msg <<- conditionMessage(w)
+      invokeRestart("muffleWarning")
       }
     ),
     error = function(e) list(error = TRUE)
@@ -253,23 +298,17 @@ test_that("mathematical correctness: variance has correct properties", {
   future::plan(future::sequential)
   set.seed(424242)
 
-  res <- bootstrap_variance(
-    dstrat, estimator,
-    point_estimate = mean(apistrat$api00),
-    bootstrap_reps = 30,
-    bootstrap_cores = 1
+  res <- suppress_bootstrap_assumption_warning(
+    bootstrap_variance(
+      dstrat, estimator,
+      point_estimate = mean(apistrat$api00),
+      bootstrap_reps = 30
+    )
   )
 
-# Property 1: Variance must be non-negative
   expect_true(res$variance >= 0)
-
-# Property 2: SE = sqrt(variance)
   expect_equal(res$se, sqrt(res$variance))
-
-# Property 3: All replicates are finite
   expect_true(all(is.finite(res$replicates)))
-
-# Property 4: Correct number of replicates
   expect_equal(length(res$replicates), 30)
 
 # Test 2: Compare to analytical ground truth for IID normal data
@@ -291,18 +330,12 @@ test_that("mathematical correctness: variance has correct properties", {
   res_iid <- bootstrap_variance(
     data_iid, estimator_mean,
     point_estimate = mean(x),
-    bootstrap_reps = 500,
-    bootstrap_cores = 1
+    bootstrap_reps = 500
   )
 
-# Bootstrap estimate should be close to analytical value
-# With 500 bootstrap reps, allow 30% tolerance due to Monte Carlo error
+# Bootstrap estimate should be close to analytical value with 500 bootstrap reps
   expect_equal(res_iid$variance, true_variance_of_mean, tolerance = 0.3)
-
-# Variance should be positive
   expect_true(res_iid$variance > 0)
-
-# SE should match
   expect_equal(res_iid$se, sqrt(res_iid$variance))
 })
 
@@ -324,24 +357,26 @@ test_that("boundary cases: minimum replicates and edge conditions", {
   future::plan(future::sequential)
 
 # Test 1: bootstrap_reps = 2 (minimum for variance calculation)
-  res_min <- bootstrap_variance(
-    dstrat, estimator,
-    point_estimate = mean(apistrat$api00),
-    bootstrap_reps = 2,
-    bootstrap_cores = 1
+  res_min <- suppress_bootstrap_assumption_warning(
+    bootstrap_variance(
+      dstrat, estimator,
+      point_estimate = mean(apistrat$api00),
+      bootstrap_reps = 2
+    )
   )
   expect_equal(length(res_min$replicates), 2)
   expect_true(is.finite(res_min$variance))
   expect_true(res_min$variance >= 0)
 
 # Test 2: bootstrap_reps = 1 should still work but variance may be problematic
-# (svrVar should handle this, but it's an edge case)
+# svrVar should handle this, but it's an edge case
   res_one <- tryCatch(
-    bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 1,
-      bootstrap_cores = 1
+    suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 1
+      )
     ),
     error = function(e) list(error = TRUE, message = e$message)
   )
@@ -358,20 +393,18 @@ test_that("boundary cases: minimum replicates and edge conditions", {
   res_small <- bootstrap_variance(
     small_data, estimator_simple,
     point_estimate = mean(small_data$y),
-    bootstrap_reps = 10,
-    bootstrap_cores = 1
+    bootstrap_reps = 10
   )
   expect_equal(length(res_small$replicates), 10)
   expect_true(is.finite(res_small$variance))
   expect_true(res_small$variance >= 0)
 
-# Test 4: Single observation (edge case)
+# Test 4: Single observation edge case
   single_data <- data.frame(y = 5)
   res_single <- bootstrap_variance(
     single_data, estimator_simple,
     point_estimate = 5,
-    bootstrap_reps = 10,
-    bootstrap_cores = 1
+    bootstrap_reps = 10
   )
 # With single observation, all bootstrap samples are identical
   expect_equal(length(res_single$replicates), 10)
@@ -421,7 +454,7 @@ test_that("omit policy correctly subsets rscales for mathematical correctness",
 # must be subsetted to the same indices for svrVar formula to be correct:
 # V = scale * sum(rscales[j] * (theta[j] - theta_0)^2)
 
-# Create controlled failure pattern with specific indices
+# Create controlled failure pattern
   fail_at <- c(2, 5, 8, 11, 14) # 5 failures out of 20
   counter <- 0
   estimator <- function(data, ...) {
@@ -438,34 +471,32 @@ test_that("omit policy correctly subsets rscales for mathematical correctness",
   set.seed(999)
 
   expect_warning(
-    res <- bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 20,
-      bootstrap_cores = 1,
-      survey_na_policy = "omit"
+    res <- suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 20,
+        survey_na_policy = "omit"
+      )
     ),
     "5/20.*failed.*omitted"
   )
 
-# Critical verification 1: Number of replicates matches expected successes
+# Number of replicates matches expected successes
   expect_equal(length(res$replicates), 15) # 20 - 5 failed
 
-# Critical verification 2: All returned replicates are finite
+# All returned replicates are finite
 # (If rscales weren't subsetted correctly, NA values might slip through)
   expect_true(all(is.finite(res$replicates)))
 
-# Critical verification 3: Variance is finite and positive
+# Variance is finite and positive
 # If rscales length didn't match replicates length, svrVar would fail or
 # produce NA/Inf, or use wrong scaling factors
   expect_true(is.finite(res$variance))
   expect_true(res$variance > 0)
-
-# Critical verification 4: SE is consistent with variance
   expect_equal(res$se, sqrt(res$variance))
 
-# Verification 5: Test with different failure patterns to ensure robustness
-# Pattern: Random failures
+# Test with different failure patterns to ensure robustness
   set.seed(777)
   fail_randomly <- sample(1:30, 12) # 12 random failures out of 30
   counter <- 0
@@ -483,12 +514,13 @@ test_that("omit policy correctly subsets rscales for mathematical correctness",
   set.seed(888)
 
   expect_warning(
-    res_random <- bootstrap_variance(
-      dstrat, estimator_random,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 30,
-      bootstrap_cores = 1,
-      survey_na_policy = "omit"
+    res_random <- suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator_random,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 30,
+        survey_na_policy = "omit"
+      )
     ),
     "12/30.*failed"
   )
@@ -527,11 +559,12 @@ test_that("survey NA policy default is 'strict'", {
 
 # Without specifying survey_na_policy, should default to strict and error
   err <- tryCatch(
-    bootstrap_variance(
-      dstrat, estimator,
-      point_estimate = mean(apistrat$api00),
-      bootstrap_reps = 10,
-      bootstrap_cores = 1
+    suppress_bootstrap_assumption_warning(
+      bootstrap_variance(
+        dstrat, estimator,
+        point_estimate = mean(apistrat$api00),
+        bootstrap_reps = 10
+      )
     ),
     error = function(e) e$message
   )
@@ -539,3 +572,148 @@ test_that("survey NA policy default is 'strict'", {
   expect_match(err, "strict")
   expect_match(err, "1/10.*failed")
 })
+
+test_that("survey bootstrap warns for multistage/FPC designs", {
+  skip_if_not_installed("survey")
+  skip_if_not_installed("svrep")
+
+  data(mu284, package = "survey")
+  multistage_design <- survey::svydesign(
+    data = mu284,
+    ids = ~ id1 + id2,
+    fpc = ~ n1 + n2
+  )
+
+  estimator <- function(data, ...) {
+    est <- survey::svymean(~y1, data)
+    list(y_hat = as.numeric(est), converged = TRUE)
+  }
+
+  future::plan(future::sequential)
+
+  expect_warning(
+    bootstrap_variance(
+      multistage_design,
+      estimator,
+      point_estimate = mean(mu284$y1),
+      bootstrap_reps = 10
+    ),
+    "injects replicate analysis weights",
+    fixed = TRUE
+  )
+})
+
+test_that("nmar.bootstrap_apply option controls future.apply usage", {
+  skip_if_not_installed("future")
+  skip_if_not_installed("future.apply")
+
+  df <- data.frame(y = c(1, 2, 3, 4))
+  estimator_mean <- function(data, ...) list(y_hat = mean(data$y), converged = TRUE)
+
+  future::plan(future::sequential)
+
+  old_opt <- getOption("nmar.bootstrap_apply", NULL)
+  on.exit(options(nmar.bootstrap_apply = old_opt), add = TRUE)
+
+  called <- new.env(parent = emptyenv())
+  called$future_lapply <- FALSE
+
+  testthat::local_mocked_bindings(
+    future_lapply = function(X, FUN, ...) {
+      called$future_lapply <- TRUE
+      lapply(X, FUN)
+    },
+    .package = "future.apply"
+  )
+
+  options(nmar.bootstrap_apply = "base")
+  res_base <- bootstrap_variance(
+    df,
+    estimator_func = estimator_mean,
+    point_estimate = mean(df$y),
+    bootstrap_reps = 5
+  )
+  expect_false(isTRUE(called$future_lapply))
+  expect_true(is.finite(res_base$variance))
+
+  called$future_lapply <- FALSE
+  options(nmar.bootstrap_apply = "future")
+  res_future <- bootstrap_variance(
+    df,
+    estimator_func = estimator_mean,
+    point_estimate = mean(df$y),
+    bootstrap_reps = 5
+  )
+  expect_true(isTRUE(called$future_lapply))
+  expect_true(is.finite(res_future$variance))
+})
+
+test_that("nmar.bootstrap_apply='auto' uses base unless parallel is configured", {
+  skip_if_not_installed("future")
+  skip_if_not_installed("future.apply")
+
+  df <- data.frame(y = c(1, 2, 3, 4))
+  estimator_mean <- function(data, ...) list(y_hat = mean(data$y), converged = TRUE)
+
+  future::plan(future::sequential)
+
+  old_opt <- getOption("nmar.bootstrap_apply", NULL)
+  on.exit(options(nmar.bootstrap_apply = old_opt), add = TRUE)
+
+  testthat::local_mocked_bindings(
+    future_lapply = function(...) stop("future_lapply should not be called in auto+sequential"),
+    .package = "future.apply"
+  )
+  options(nmar.bootstrap_apply = "auto")
+
+  res_auto <- bootstrap_variance(
+    df,
+    estimator_func = estimator_mean,
+    point_estimate = mean(df$y),
+    bootstrap_reps = 5
+  )
+  expect_true(is.finite(res_auto$variance))
+
+  called <- new.env(parent = emptyenv())
+  called$future_lapply <- FALSE
+
+  testthat::local_mocked_bindings(
+    future_lapply = function(X, FUN, ...) {
+      called$future_lapply <- TRUE
+      lapply(X, FUN)
+    },
+    .package = "future.apply"
+  )
+  testthat::local_mocked_bindings(
+    nmar_future_workers = function() 2L,
+    .package = "NMAR"
+  )
+
+  res_auto_parallel <- bootstrap_variance(
+    df,
+    estimator_func = estimator_mean,
+    point_estimate = mean(df$y),
+    bootstrap_reps = 5
+  )
+  expect_true(isTRUE(called$future_lapply))
+  expect_true(is.finite(res_auto_parallel$variance))
+})
+
+test_that("nmar.bootstrap_apply validates option value", {
+  df <- data.frame(y = c(1, 2, 3, 4))
+  estimator_mean <- function(data, ...) list(y_hat = mean(data$y), converged = TRUE)
+
+  old_opt <- getOption("nmar.bootstrap_apply", NULL)
+  on.exit(options(nmar.bootstrap_apply = old_opt), add = TRUE)
+  options(nmar.bootstrap_apply = "nope")
+  expect_error(
+    bootstrap_variance(
+      df,
+      estimator_func = estimator_mean,
+      point_estimate = mean(df$y),
+      bootstrap_reps = 5
+    ),
+    "nmar.bootstrap_apply",
+    fixed = TRUE
+  )
+})
